@@ -23,6 +23,7 @@ class _EmbossMinimalCommandLine(AbstractCommandline):
     This is provided for subclassing, it deals with shared options
     common to all the EMBOSS tools:
 
+    Attributes:
      - auto               Turn off prompts
      - stdout             Write standard output
      - filter             Read standard input, write standard output
@@ -36,7 +37,9 @@ class _EmbossMinimalCommandLine(AbstractCommandline):
      - error              Report errors
      - fatal              Report fatal errors
      - die                Report dying program messages
+
     """
+
     def __init__(self, cmd=None, **kwargs):
         assert cmd is not None
         extra_parameters = [
@@ -93,6 +96,7 @@ class _EmbossCommandLine(_EmbossMinimalCommandLine):
      - outfile            Output filename
 
     """
+
     def __init__(self, cmd=None, **kwargs):
         assert cmd is not None
         extra_parameters = [
@@ -151,7 +155,9 @@ class Primer3Commandline(_EmbossCommandLine):
     eprimer3 -auto -outfile=myresults.out -sequence=mysequence.fas -hybridprobe=True -productosize=200 -oligosize=20 -explainflag=True
 
     """
+
     def __init__(self, cmd="eprimer3", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "Sequence to choose primers from.",
@@ -457,6 +463,7 @@ class PrimerSearchCommandline(_EmbossCommandLine):
     """Commandline object for the primersearch program from EMBOSS."""
 
     def __init__(self, cmd="primersearch", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-seqall", "-sequences", "sequences", "seqall"],
                     "Sequence to look for the primer pairs in.",
@@ -488,7 +495,9 @@ class FDNADistCommandline(_EmbossCommandLine):
     fdnadist is an EMBOSS wrapper for the PHYLIP program dnadist for
     calulating distance matrices from DNA sequence files.
     """
+
     def __init__(self, cmd="fdnadist", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "seq file to use (phylip)",
@@ -529,7 +538,9 @@ class FTreeDistCommandline(_EmbossCommandLine):
     ftreedist is an EMBOSS wrapper for the PHYLIP program treedist used for
     calulating distance measures between phylogentic trees.
     """
+
     def __init__(self, cmd="ftreedist", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-intreefile", "intreefile"],
                     "tree file to score (phylip)",
@@ -555,7 +566,9 @@ class FNeighborCommandline(_EmbossCommandLine):
     fneighbor is an EMBOSS wrapper for the PHYLIP program neighbor used for
     calulating neighbor-joining or UPGMA trees from distance matrices.
     """
+
     def __init__(self, cmd="fneighbor", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-datafile", "datafile"],
                     "dist file to use (phylip)",
@@ -589,7 +602,9 @@ class FSeqBootCommandline(_EmbossCommandLine):
     fseqboot is an EMBOSS wrapper for the PHYLIP program seqboot used to
     pseudo-sample alignment files.
     """
+
     def __init__(self, cmd="fseqboot", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "seq file to sample (phylip)",
@@ -632,7 +647,9 @@ class FDNAParsCommandline(_EmbossCommandLine):
     "interactive mode" (and as a result fail if called with subprocess) if
     "-auto" is not set to true.
     """
+
     def __init__(self, cmd="fdnapars", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "seq file to use (phylip)",
@@ -679,7 +696,9 @@ class FProtParsCommandline(_EmbossCommandLine):
     "interactive mode" (and as a result fail if called with subprocess) if
     "-auto" is not set to true.
     """
+
     def __init__(self, cmd="fprotpars", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "seq file to use (phylip)",
@@ -719,7 +738,9 @@ class FProtDistCommandline(_EmbossCommandLine):
     fprotdist is an EMBOSS wrapper for the PHYLIP program protdist used to
     estimate trees from protein sequences using parsimony
     """
+
     def __init__(self, cmd="fprotdist", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "seq file to use (phylip)",
@@ -761,7 +782,9 @@ class FConsenseCommandline(_EmbossCommandLine):
     fconsense is an EMBOSS wrapper for the PHYLIP program consense used to
     calculate consensus trees.
     """
+
     def __init__(self, cmd="fconsense", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-intreefile", "intreefile"],
                     "file with phylip trees to make consensus from",
@@ -784,9 +807,10 @@ class FConsenseCommandline(_EmbossCommandLine):
 
 
 class WaterCommandline(_EmbossCommandLine):
-    """Commandline object for the water program from EMBOSS.
-    """
+    """Commandline object for the water program from EMBOSS."""
+
     def __init__(self, cmd="water", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-asequence", "asequence"],
                     "First sequence to align",
@@ -823,7 +847,9 @@ class WaterCommandline(_EmbossCommandLine):
 
 class NeedleCommandline(_EmbossCommandLine):
     """Commandline object for the needle program from EMBOSS."""
+
     def __init__(self, cmd="needle", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-asequence", "asequence"],
                     "First sequence to align",
@@ -869,6 +895,7 @@ class NeedleallCommandline(_EmbossCommandLine):
     """Commandline object for the needleall program from EMBOSS."""
 
     def __init__(self, cmd="needleall", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-asequence", "asequence"],
                     "First sequence to align",
@@ -918,6 +945,7 @@ class StretcherCommandline(_EmbossCommandLine):
     """Commandline object for the stretcher program from EMBOSS."""
 
     def __init__(self, cmd="stretcher", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-asequence", "asequence"],
                     "First sequence to align",
@@ -952,6 +980,7 @@ class FuzznucCommandline(_EmbossCommandLine):
     """Commandline object for the fuzznuc program from EMBOSS."""
 
     def __init__(self, cmd="fuzznuc", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "Sequence database USA",
@@ -959,9 +988,8 @@ class FuzznucCommandline(_EmbossCommandLine):
             _Option(["-pattern", "pattern"],
                     "Search pattern, using standard IUPAC one-letter codes",
                     is_required=True),
-            _Option(["-mismatch", "mismatch"],
-                    "Number of mismatches",
-                    is_required=True),
+            _Option(["-pmismatch", "pmismatch"],
+                    "Number of mismatches"),
             _Option(["-complement", "complement"],
                     "Search complementary strand"),
             _Option(["-rformat", "rformat"],
@@ -970,10 +998,31 @@ class FuzznucCommandline(_EmbossCommandLine):
         _EmbossCommandLine.__init__(self, cmd, **kwargs)
 
 
+class FuzzproCommandline(_EmbossCommandLine):
+    """Commandline object for the fuzzpro program from EMBOSS."""
+
+    def __init__(self, cmd="fuzzpro", **kwargs):
+        """Initialize the class."""
+        self.parameters = [
+            _Option(["-sequence", "sequence"],
+                    "Sequence database USA",
+                    is_required=True),
+            _Option(["-pattern", "pattern"],
+                    "Search pattern, using standard IUPAC one-letter codes",
+                    is_required=True),
+            _Option(["-pmismatch", "pmismatch"],
+                    "Number of mismatches"),
+            _Option(["-rformat", "rformat"],
+                    "Specify the report format to output in."),
+            ]
+        _EmbossCommandLine.__init__(self, cmd, **kwargs)
+
+
 class Est2GenomeCommandline(_EmbossCommandLine):
     """Commandline object for the est2genome program from EMBOSS."""
 
     def __init__(self, cmd="est2genome", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-est", "est"],
                     "EST sequence(s)",
@@ -1022,6 +1071,7 @@ class ETandemCommandline(_EmbossCommandLine):
     """Commandline object for the etandem program from EMBOSS."""
 
     def __init__(self, cmd="etandem", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "Sequence",
@@ -1049,6 +1099,7 @@ class EInvertedCommandline(_EmbossCommandLine):
     """Commandline object for the einverted program from EMBOSS."""
 
     def __init__(self, cmd="einverted", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "Sequence",
@@ -1077,6 +1128,7 @@ class PalindromeCommandline(_EmbossCommandLine):
     """Commandline object for the palindrome program from EMBOSS."""
 
     def __init__(self, cmd="palindrome", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "Sequence",
@@ -1105,6 +1157,7 @@ class TranalignCommandline(_EmbossCommandLine):
     """Commandline object for the tranalign program from EMBOSS."""
 
     def __init__(self, cmd="tranalign", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-asequence", "asequence"],
                     "Nucleotide sequences to be aligned.",
@@ -1128,6 +1181,7 @@ class DiffseqCommandline(_EmbossCommandLine):
     """Commandline object for the diffseq program from EMBOSS."""
 
     def __init__(self, cmd="diffseq", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-asequence", "asequence"],
                     "First sequence to compare",
@@ -1157,8 +1211,8 @@ class DiffseqCommandline(_EmbossCommandLine):
 class IepCommandline(_EmbossCommandLine):
     """Commandline for EMBOSS iep: calculated isoelectric point and charge.
 
-    Example:
-
+    Examples
+    --------
     >>> from Bio.Emboss.Applications import IepCommandline
     >>> iep_cline = IepCommandline(sequence="proteins.faa",
     ...                            outfile="proteins.txt")
@@ -1167,8 +1221,11 @@ class IepCommandline(_EmbossCommandLine):
 
     You would typically run the command line with iep_cline() or via the
     Python subprocess module, as described in the Biopython tutorial.
+
     """
+
     def __init__(self, cmd="iep", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "Protein sequence(s) filename",
@@ -1213,7 +1270,9 @@ class SeqretCommandline(_EmbossMinimalCommandLine):
     This wrapper currently only supports the core functionality, things like
     feature tables (in EMBOSS 6.1.0 onwards) are not yet included.
     """
+
     def __init__(self, cmd="seqret", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "Input sequence(s) filename",
@@ -1242,7 +1301,7 @@ class SeqretCommandline(_EmbossMinimalCommandLine):
 
 
 class SeqmatchallCommandline(_EmbossCommandLine):
-    """ Commandline object for the seqmatchall program from EMBOSS
+    """Commandline object for the seqmatchall program from EMBOSS.
 
     e.g.
     >>> cline = SeqmatchallCommandline(sequence="opuntia.fasta", outfile="opuntia.txt")
@@ -1253,7 +1312,9 @@ class SeqmatchallCommandline(_EmbossCommandLine):
     seqmatchall -auto -outfile=opuntia.txt -sequence=opuntia.fasta -wordsize=18 -aformat=pair
 
     """
+
     def __init__(self, cmd="seqmatchall", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             _Option(["-sequence", "sequence"],
                     "Readable set of sequences",
